///////////////////////////////////////////////
// Pilotage extracteur à miel
// Arduino Nano
// rev. 1.00 : 06/2020
// rev. 2.00 : 02/2024 - adaptation programme pour hardware V2 (arduino pro mini)

// alex.pruvost@laposte.net
///////////////////////////////////////////////

#include <Arduino.h>

// Version à compiler :
// #define HARDWAREV1
#define HARDWAREV2

// #define modeMiseAuPoint // Décommenter pour activer la sortie série et les fonctions de mise au point

// ---------------------------------
// Constantes de calibrage du système de pilotage :

const float COEFFACCELERATIONMAX = 0.2;
const float COEFFFREINAGESTANSARDFIXE = 0.4;

//const uint8_t TALONVITESSEMINIMALEDEDEMARRAGE = 40; // Hardware V1
const uint8_t TALONVITESSEMINIMALEDEDEMARRAGE = 50; // Hardware V2

const float COEFFACCELERATIONAUTOMIN = 0.01; // Potar en position mini
const uint8_t VITESSEAUTOMIN = 60;           // Potar en position mini
const uint8_t DIVISEURDECOMPTAGEPOURDUREEAUTO = 10;

// ---------------------------------
// Assignation des entrées/sorties :

#ifdef HARDWAREV1

const uint8_t ledPin = 13;
const uint8_t BOUTONMANUELAUTO = 2;
const uint8_t BOUTONMARCHEARRET = 3;

const uint8_t MOSFET = 11; // Défini par TIMER2 PWM, peut pas changer

const uint8_t LEDPHASEACCELERATION = 12;
const uint8_t LEDPHASEFREINAGE = 4;
const uint8_t LEDCYCLELANCE = 19;

const uint8_t LEDPROGRESSION1 = 5;
const uint8_t LEDPROGRESSION2 = 7;
const uint8_t LEDPROGRESSION3 = 6;
const uint8_t LEDPROGRESSION4 = 8;
const uint8_t LEDPROGRESSION5 = 9;
const uint8_t LEDPROGRESSION6 = 10;

const uint8_t POTARVITESSEMANUEL = A0;
const uint8_t POTARACCELERATION = A1;
const uint8_t POTARVITESSEAUTO = A6;
const uint8_t POTARDUREEAUTO = A7;

#endif

#ifdef HARDWAREV2

const uint8_t ledPin = 13;
const uint8_t BOUTONMANUELAUTO = 3;
const uint8_t BOUTONMARCHEARRET = 2;

const uint8_t MOSFET = 11; // Défini par TIMER2 PWM, peut pas changer

const uint8_t LEDPHASEACCELERATION = 5;
const uint8_t LEDPHASEFREINAGE = 10;
const uint8_t LEDCYCLELANCE = 4;

const uint8_t LEDPROGRESSION6 = 6;
const uint8_t LEDPROGRESSION5 = 7;
const uint8_t LEDPROGRESSION4 = 8;
const uint8_t LEDPROGRESSION3 = 9;
const uint8_t LEDPROGRESSION2 = 13;
const uint8_t LEDPROGRESSION1 = 12;

const uint8_t POTARVITESSEMANUEL = A3;
const uint8_t POTARACCELERATION = A0;
const uint8_t POTARVITESSEAUTO = A1;
const uint8_t POTARDUREEAUTO = A2; // Valeur du potentiomètre directement en secondes (0 à 1024 secondes de durée)

#endif

const uint8_t LEDSDELABANDE[] = {LEDPROGRESSION1, LEDPROGRESSION2, LEDPROGRESSION3, LEDPROGRESSION4, LEDPROGRESSION5, LEDPROGRESSION6};

// Modes de fonctionnement :
enum
{
  cycleArrete,
  cycleModeManuel,
  cycleFreinage,
  cycleAutoAcceleration,
  cycleAutoStabilise
};

// ---------------------------------
// Variables globales :
bool interruptionTimer1 = false;

uint16_t vitesseManuel, acceleration, vitesseAuto = 0;
uint32_t timeStampDepartComptage, delaiEntreChaqueLED = 0;

float vitesseActuelle = 0; // Sert à mémoriser la vitesse en cours, pour ne jamais passer brutalement de la vitesse max à la vitesse zéro, sinon casse moteur
float vitesseAappliquer, coeffAcceleration, vitesseAutoLueSurPotar = 0;

uint8_t comptageNombreDeBoucles, avancementCycleAuto = 0;
int valeurActuelle = 0;

bool estCeQueLeBoutonMarcheArretAeteRelache, premierTourDeBoucle = true;

uint8_t mode = cycleArrete;

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// Définitions :

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
void setup()
{
  // put your setup code here, to run once:

  pinMode(ledPin, OUTPUT);

  pinMode(LEDPHASEACCELERATION, OUTPUT);
  pinMode(LEDPHASEFREINAGE, OUTPUT);
  pinMode(LEDCYCLELANCE, OUTPUT);
  pinMode(MOSFET, OUTPUT);

  pinMode(LEDPROGRESSION1, OUTPUT);
  pinMode(LEDPROGRESSION2, OUTPUT);
  pinMode(LEDPROGRESSION3, OUTPUT);
  pinMode(LEDPROGRESSION4, OUTPUT);
  pinMode(LEDPROGRESSION5, OUTPUT);
  pinMode(LEDPROGRESSION6, OUTPUT);

  pinMode(BOUTONMANUELAUTO, INPUT_PULLUP);
  pinMode(BOUTONMARCHEARRET, INPUT_PULLUP);

  pinMode(POTARVITESSEMANUEL, INPUT);
  pinMode(POTARACCELERATION, INPUT);
  pinMode(POTARVITESSEAUTO, INPUT);
  pinMode(POTARDUREEAUTO, INPUT);

  analogReference(DEFAULT);

  // open the serial port
  Serial.begin(115200);

  // Réglage TIMER1 pour interruption par ovfw toutes les 1/32 secondes
  cli(); // stop interrupts
  TCCR1A = 0;
  TCCR1B = 0;
  TCCR1C = 0;
  TCCR1B |= (1 << CS11);
  TIMSK1 |= (1 << TOIE1);
  sei(); // allow interrupts

  //  TIMER2 désactivé (commenté) pour la simulation avec Proteus, sinon ralentit trop la simulation
  // Réglage TIMER2 pour PWM @ 31,25 KHz  - OCR2A
  cli(); // stop interrupts
  TCCR2A = 0;
  TCCR2B = 0;
  TCCR2A |= (1 << COM2A1); // Non inverting mode : 1 à BOTTOM, 0 à MATCH
  TCCR2A |= (1 << WGM20);  // Fast PWM phase correct mode 1
  TCCR2B |= (1 << CS20);   // ClK / no prescaling
  TIMSK2 = 0;
  sei(); // allow interrupts

  digitalWrite(LEDPHASEACCELERATION, HIGH);
  digitalWrite(LEDPHASEFREINAGE, HIGH);
  digitalWrite(LEDCYCLELANCE, HIGH);
  digitalWrite(LEDPROGRESSION1, HIGH);
  digitalWrite(LEDPROGRESSION2, HIGH);
  digitalWrite(LEDPROGRESSION3, HIGH);
  digitalWrite(LEDPROGRESSION4, HIGH);
  digitalWrite(LEDPROGRESSION5, HIGH);
  digitalWrite(LEDPROGRESSION6, HIGH);

  delay(2000);

  digitalWrite(LEDPHASEACCELERATION, LOW);
  digitalWrite(LEDPHASEFREINAGE, LOW);
  digitalWrite(LEDCYCLELANCE, LOW);
  digitalWrite(LEDPROGRESSION1, LOW);
  digitalWrite(LEDPROGRESSION2, LOW);
  digitalWrite(LEDPROGRESSION3, LOW);
  digitalWrite(LEDPROGRESSION4, LOW);
  digitalWrite(LEDPROGRESSION5, LOW);
  digitalWrite(LEDPROGRESSION6, LOW);

} // end setup

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// Vecteurs d'interruption

ISR(TIMER1_OVF_vect)
{
  interruptionTimer1 = true;
}

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
void loop()
{

  // ---------------------------------
  // Variables :

  if (interruptionTimer1)
  {
    interruptionTimer1 = false;

    vitesseManuel = analogRead(POTARVITESSEMANUEL);
    acceleration = analogRead(POTARACCELERATION);

    vitesseAutoLueSurPotar = analogRead(POTARVITESSEAUTO) / 4;
    vitesseAuto = TALONVITESSEMINIMALEDEDEMARRAGE + vitesseAutoLueSurPotar * (256 - TALONVITESSEMINIMALEDEDEMARRAGE) / 256; // Calibrage du potar entre la vitesse min et max

    ++comptageNombreDeBoucles;

#ifdef modeMiseAuPoint
    // Mode Debug ----------------------------
    if (comptageNombreDeBoucles == 32)
    { // Rafraichissement valeurs affichées toutes les secondes
      Serial.write(12);
      Serial.print("Pilotage moteur extracteur - Mode mise au point -");
      Serial.println("--------------------------------------------");
      Serial.print("POTARVITESSEMANUEL = ");
      Serial.println(vitesseManuel);
      Serial.print("POTARACCELERATION = ");
      Serial.println(acceleration);
      Serial.print("POTARVITESSEAUTO = ");
      Serial.println(vitesseAuto);
      Serial.print("POTARDUREEAUTO = ");
      Serial.print(dureeAuto);
      Serial.print(" / ");
      Serial.println(comptageNombreDeBouclesPourModeAuto / DIVISEURDECOMPTAGEPOURDUREEAUTO);
      Serial.print("BOUTONMANUELAUTO = ");
      Serial.println(digitalRead(BOUTONMANUELAUTO));
      Serial.print("BOUTONMARCHEARRET = ");
      Serial.println(digitalRead(BOUTONMARCHEARRET));
      Serial.print("mode = ");
      Serial.println(mode);
      Serial.print("valeur PWM sur 255 = ");
      Serial.println(OCR2A);
      Serial.println("Etat des LEDs ------------------------------");
      Serial.print("LEDCYCLELANCE = ");
      Serial.println(digitalRead(LEDCYCLELANCE));

      Serial.print("LEDPHASEACCELERATION = ");
      Serial.println(digitalRead(LEDPHASEACCELERATION));

      Serial.print("LEDPROGRESSION = ");
      Serial.print(digitalRead(LEDPROGRESSION1));
      Serial.print(digitalRead(LEDPROGRESSION2));
      Serial.print(digitalRead(LEDPROGRESSION3));
      Serial.print(digitalRead(LEDPROGRESSION4));
      Serial.print(digitalRead(LEDPROGRESSION5));
      Serial.println(digitalRead(LEDPROGRESSION6));

      Serial.print("LEDPHASEFREINAGE = ");
      Serial.println(digitalRead(LEDPHASEFREINAGE));

      Serial.println("--------------------------------------------");
      comptageNombreDeBoucles = 0;
      //      digitalWrite(ledPin, !digitalRead(ledPin));
    }
#endif

    /*  Modes de fonctionnement :

    0   Cycle arrêté
    1   Cycle lancé mode manuel
    2   Cycle en cours de freinage
    3   Cycle lancé auto en phase d'accélération
    4   Cycle lancé auto en phase stable
     */

    // Gestion lancement cycle par les boutons ------------------

    if (digitalRead(BOUTONMARCHEARRET))
    {
      estCeQueLeBoutonMarcheArretAeteRelache = true;
    }

    if (!digitalRead(BOUTONMARCHEARRET) && mode == cycleArrete)
    {

#ifdef HARDWAREV1
      digitalRead(BOUTONMANUELAUTO) ? mode = cycleModeManuel : mode = cycleAutoAcceleration;
#endif
#ifdef HARDWAREV2
      digitalRead(BOUTONMANUELAUTO) ? mode = cycleAutoAcceleration : mode = cycleModeManuel;
#endif

      estCeQueLeBoutonMarcheArretAeteRelache = false;
      digitalWrite(LEDCYCLELANCE, HIGH);
    }

    if (!digitalRead(BOUTONMARCHEARRET) && mode != cycleArrete && estCeQueLeBoutonMarcheArretAeteRelache) // Interrompre le cycle en cours
    {
      mode = cycleFreinage;
      digitalWrite(LEDPHASEFREINAGE, HIGH);
      digitalWrite(LEDPHASEACCELERATION, LOW);
      avancementCycleAuto = 0;
    }

    if (mode == cycleArrete && vitesseActuelle == 0)
    {
      // digitalWrite(LEDCYCLELANCE,LOW);
      digitalWrite(LEDPHASEFREINAGE, LOW);

      // Clignotement de base de la led départ cycle
      if (comptageNombreDeBoucles >= 32)
      { // Rafraichissement valeurs affichées toutes les secondes
        comptageNombreDeBoucles = 0;
        digitalWrite(LEDCYCLELANCE, !digitalRead(LEDCYCLELANCE));
      }
    }

    if (mode == cycleModeManuel)
    {

      if (vitesseManuel / 4 > vitesseActuelle)
      {
        vitesseAappliquer = vitesseActuelle + COEFFACCELERATIONMAX; // vitesseAappliquer est un float
      }
      else
      {
        vitesseAappliquer = vitesseActuelle - COEFFFREINAGESTANSARDFIXE; // vitesseAappliquer est un float
      }

      OCR2A = (int)vitesseAappliquer; // On écrête vitesseAappliquer avec la fonction int

      vitesseActuelle = vitesseAappliquer;
    }

    if (mode == cycleAutoAcceleration)
    {
      digitalWrite(LEDPHASEACCELERATION, HIGH);

      if (vitesseActuelle < TALONVITESSEMINIMALEDEDEMARRAGE)
      {
        vitesseActuelle = TALONVITESSEMINIMALEDEDEMARRAGE;
      }
      if (vitesseAuto > vitesseActuelle)
      {

        coeffAcceleration = COEFFACCELERATIONAUTOMIN + acceleration * (COEFFACCELERATIONMAX - COEFFACCELERATIONAUTOMIN) / 1024; // Calibrage du potar entre l'accel min et max
        vitesseAappliquer = vitesseActuelle + coeffAcceleration;                                                                // vitesseAappliquer est un float
      }
      else
      {
        mode = cycleAutoStabilise;
        digitalWrite(LEDPHASEACCELERATION, LOW);
      }

      OCR2A = (int)vitesseAappliquer; // On écrête vitesseAappliquer avec la fonction int
      vitesseActuelle = vitesseAappliquer;
    }

    if (mode == cycleAutoStabilise)
    {

      switch (avancementCycleAuto)
      {
      case 0:
        delaiEntreChaqueLED = map(analogRead(POTARDUREEAUTO), 0, 1023, 0, 360000/6);
        timeStampDepartComptage = millis() + delaiEntreChaqueLED;

        avancementCycleAuto = 7;
        for (uint8_t i = 0; i < 6; i++)
          digitalWrite(LEDSDELABANDE[i], HIGH);
        break;

      default:
        if (millis() > timeStampDepartComptage)
        {
          avancementCycleAuto--;
          timeStampDepartComptage = timeStampDepartComptage + delaiEntreChaqueLED;

          for (uint8_t i = 0; i < 6; i++)
            ((avancementCycleAuto - 1) > i) ? digitalWrite(LEDSDELABANDE[i], HIGH) : digitalWrite(LEDSDELABANDE[i], LOW);
        }
        break;

      case 1:
        mode = cycleFreinage;
        digitalWrite(LEDPROGRESSION6, LOW);
        digitalWrite(LEDPHASEFREINAGE, HIGH);
        avancementCycleAuto = 0;
        break;
      }
    }

    if (mode == cycleFreinage)
    {
      vitesseAappliquer = vitesseActuelle - COEFFFREINAGESTANSARDFIXE; // vitesseAappliquer est un float

      for (uint8_t i = 0; i < 6; i++)
        digitalWrite(LEDSDELABANDE[i], LOW);

      if (vitesseAappliquer < 0)
      {
        vitesseAappliquer = 0;
        mode = cycleArrete;
      }

      OCR2A = (int)vitesseAappliquer; // On écrête vitesseAappliquer avec la fonction int
      vitesseActuelle = vitesseAappliquer;
    }
  }
}

//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// Fonctions et méthodes
